@@ -1,11 +1,14 @@
 import os
 import shutil
+import re
 from pathlib import Path
+from functools import lru_cache
 
 from django.core.management import call_command
 from django_test_migrations.migrator import Migrator
 from django.contrib.auth import get_user_model
 from django.contrib.staticfiles.testing import StaticLiveServerTestCase
+from django.db import connections
 from django.urls import reverse
 from django.apps import apps
 
@@ -13,6 +16,64 @@ from playwright.sync_api import sync_playwright, expect
 from polymorphic import tests
 
 
+DSN_RE = re.compile(r"^(?P<host>[^:/]+)(:(?P<port>\d+))?/(?P<service>.+)$")
+
+
+@lru_cache()
+def is_sqlite_in_memory(db_name: str = "default") -> bool:
+    from django.conf import settings
+
+    return (
+        settings.DATABASES[db_name]["ENGINE"] == "django.db.backends.sqlite3"
+        and settings.DATABASES[db_name]["NAME"] == ":memory:"
+    )
+
+
+@lru_cache()
+def is_oracle(db_name: str = "default") -> bool:
+    from django.conf import settings
+
+    return settings.DATABASES[db_name]["ENGINE"] == "django.db.backends.oracle"
+
+
+def get_subprocess_test_db_env(db_name: str = "default") -> dict[str, str]:
+    """
+    If you need to run a test in a subprocess that accesses the active test database
+    (e.g. to call management commands), you need to set up the environment variables
+    so that the subprocess can connect to the correct test database.
+    This function returns a copy of os.environ with the necessary variables set.
+    """
+    env = os.environ.copy()
+    db = connections[db_name].settings_dict
+    # this is where django's renaming of test databases gets very annoying - we need
+    # to make sure our subprocess invocation uses the test database - which it wont
+    # do by default because it thinks we aren't in test mode.
+    if is_oracle(db_name):
+        dsn = db["NAME"]
+        m = DSN_RE.match(dsn)
+
+        if not m:
+            raise AssertionError(
+                f"Can't parse Oracle DSN from NAME={dsn!r}. "
+                "Expected format like 'host:1521/service' or 'host/service'."
+            )
+
+        host = m.group("host")
+        port = m.group("port") or "1521"
+        service = m.group("service")
+
+        env["ORACLE_DATABASES"] = service
+        env["ORACLE_USER"] = db["USER"]
+        env["ORACLE_PASSWORD"] = db["PASSWORD"]
+
+        # Only set non-empty values
+        env["ORACLE_HOST"] = host
+        env["ORACLE_PORTS"] = port
+    else:
+        env["PYTEST_DB_NAME"] = db["NAME"]
+    return env
+
+
 class GeneratedMigrationsPerClassMixin:
     """
     Generates migrations at class setup, applies them, and rolls them back at teardown.
